/*
 * Sixth core user interface. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */

package eu.svjatoslav.sixth.core;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.geometry.Rectangle;
import eu.svjatoslav.sixth.e3d.gui.Avatar;
import eu.svjatoslav.sixth.e3d.gui.ViewFrame;
import eu.svjatoslav.sixth.e3d.gui.ViewPanel;
import eu.svjatoslav.sixth.e3d.gui.textEditorComponent.TextEditComponent;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.Color;
import eu.svjatoslav.sixth.e3d.renderer.raster.ShapeCollection;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.wireframe.Grid2D;

public class Main {

    /**
     * Creates nice looking neon grid. Makes it easier to navigate in space.
     *
     * @return grid
     */
    private static Grid2D createGrid() {
        final Transform location = new Transform(
                new Point3D(0, 100, 0),
                0,
                Math.PI / 2);

        return new Grid2D(location,
                new Rectangle(2000),
                10,
                10,
                new LineAppearance(10, new Color("00b3ad")));
    }

    public static void main(final String[] args) {

        final ViewFrame viewFrame = new ViewFrame();
        final ViewPanel viewPanel = viewFrame.getViewPanel();

        final ShapeCollection rootShapeCollection = viewFrame.getViewPanel()
                .getRootShapeCollection();

        rootShapeCollection.addShape(createGrid());
        rootShapeCollection.addShape(createTextEditor(viewPanel));

        setAvatarStartLocation(viewPanel.getAvatar());
    }

    private static TextEditComponent createTextEditor(ViewPanel viewPanel) {
        return new TextEditComponent(
                    new Transform(new Point3D(0, 0, 0)), viewPanel,
                    new Point2D(200, 120));
    }

    private static void setAvatarStartLocation(Avatar avatar) {
        avatar.setLocation(new Point3D(500, -300, -800));
        avatar.setAngleXZ(0.6);
        avatar.setAngleYZ(-0.5);
    }
}
